---
title: Debian Install
description: How to install `veilid-server` and `veilid-cli` on Debian based systems
weight: 3
layout: subpage
---

### Step 1

Add the GPG keys to your operating systems keyring.

_Explanation_: The wget command downloads the public key, and the sudo gpg command adds the public key to the keyring.

<div class="code-snippet">
  <input aria-label="Get Keys" type="text" readonly value="wget -O- https://packages.veilid.net/gpg/veilid-packages-key.public | sudo gpg --dearmor -o /usr/share/keyrings/veilid-packages-keyring.gpg">
  <button class="btn btn-secondary"><span>Copy</span></button>
</div>

### Step 2

Identify your architecture

_Explanation_: The following command will tell you what type of CPU your system is running

<div class="code-snippet">
  <input aria-label="Add Keys" type="text" readonly value='dpkg --print-architecture'>
  <button class="btn btn-secondary"><span>Copy</span></button>
</div>

### Step 3

Add Veilid to your list of available software.

_Explanation_: Using the command in Step 2 you will need to run one of the following:

- For **AMD**64 based systems run this command:

<div class="code-snippet">
  <input aria-label="Add Keys" type="text" readonly value='echo "deb [arch=amd64 signed-by=/usr/share/keyrings/veilid-packages-keyring.gpg] https://packages.veilid.net/apt stable main" | sudo tee /etc/apt/sources.list.d/veilid.list 1>/dev/null'>
  <button class="btn btn-secondary"><span>Copy</span></button>
</div>

- For **ARM**64 based systems run this command:

<div class="code-snippet">
  <input aria-label="Add Keys" type="text" readonly value='echo "deb [arch=arm64 signed-by=/usr/share/keyrings/veilid-packages-keyring.gpg] https://packages.veilid.net/apt stable main" | sudo tee /etc/apt/sources.list.d/veilid.list 1>/dev/null'>
  <button class="btn btn-secondary"><span>Copy</span></button>
</div>

_Explanation_: Each of the above commands will create a new file called `veilid.list` in the `/etc/apt/sources.list.d/`. This file contains instructions that tell the operating system where to download Veilid.

### Step 4

Refresh the package manager.

_Explanation_: This tells the `apt` package manager to rebuild the list of available software using the files in `/etc/apt/sources.list.d/` directory

<div class="code-snippet">
  <input aria-label="Run Updates" type="text" readonly value="apt update">
  <button class="btn btn-secondary"><span>Copy</span></button>
</div>

#### Step 5

Install Veilid.

_Explanation_: With the package manager updated, it is now possible to install Veilid!

<div class="code-snippet">
  <input aria-label="Install the headless and command line utilities" type="text" readonly value="apt install veilid-server veilid-cli">
  <button class="btn btn-secondary"><span>Copy</span></button>
</div>

<hr />

The remaining steps are optional.

### Step 6

Start `veilid-server.service` manually

<div class="code-snippet">
  <input aria-label="Install the headless and command line utilities" type="text" readonly value="sudo systemctl start veilid-server.service">
  <button class="btn btn-secondary"><span>Copy</span></button>
</div>

### Step 7

Set up `veilid-server.service` to start at boot

<div class="code-snippet">
  <input aria-label="Install the headless and command line utilities" type="text" readonly value="sudo systemctl enable --now veilid-server.service">
  <button class="btn btn-secondary"><span>Copy</span></button>
</div>




